# FILE: config.tcl
#

Oc_Class Oc_Config {

    common configDir
    common namesDir
    common cacheDir

    # The Oc_Config instance chosen which represents the platform on which 
    # this program is running
    common runPlatform

    ClassConstructor {
        set configDir [file join [file dirname [file dirname [file dirname \
                [Oc_DirectPathname [info script]]]]] config]
        set namesDir [file join $configDir names]
        set cacheDir [file join $configDir cache]
        foreach fn [glob -nocomplain [file join $namesDir *.tcl]] {
            if {[file readable $fn]} {
                if {[catch {uplevel #0 source [list $fn]} msg]} {
                    Oc_Log Log "Error sourcing $fn:
	$msg" warning Oc_Config
                }
            }
        }
        set detected {}
        foreach i [$class Instances] {
            if {[$i Detect]} {
                lappend detected $i
            }
        }
        # The default case here should be to provide for user selection
        # of the correct configuration name.  For now, just die.
        switch -exact -- [llength $detected] {
            0 {set runPlatform [Oc_Config New _ unknown {return 1}]}
            1 {set runPlatform [lindex $detected 0]}
            default {
                set msg ""
                foreach c $detected {lappend msg [$c GetValue platform_name]}
                error "Multiple platform names are compatible with your computer:
	$msg"
            }
        }
        $runPlatform LoadTclTkCache
        $runPlatform LoadCache
    }

    proc Lookup {name} {
        foreach i [$class Instances] {
            if {[string match $name [$i GetValue platform_name]]} {
                return $i
            }
        }
        return -code error "Unknown configuration name: $name"
    }

    proc RunPlatform {} {runPlatform} {
        return $runPlatform
    }

    private variable name
    private variable detectScript

    private array variable values

    Constructor {_name _detectScript} {
        set name $_name
        if {![info complete $_detectScript]} {
            error "detectScript argument must be a complete Tcl script"
        }
        set detectScript $_detectScript
        $this SetValue platform_name $name
    }

    private method Detect {} {detectScript} {
        eval $detectScript
    }

    method GetValue {feature} {
        # Do we already have a value?
        if {[info exists values($feature)]} {
            return $values($feature)
        }

        return -code error "Unknown feature '$feature' in configuration '$name'

	Have you edited oommf/config/cache/$name.tcl ?
"
    }

    method LoadTclTkCache {} {
        global tcl_platform
        if {![string match unix $tcl_platform(platform)]} {
            return
        }
        foreach tail {tclConfig.sh tkConfig.sh} {
            foreach dir [list [file join [info library] ..] \
                    [file join [file dirname [info nameofexecutable]] .. lib]] {
                set fn [file join $dir $tail]
                if {[file readable $fn]} {
                    set f [open $fn r]
                    gets $f line
                    while {![eof $f]} {
                        if {[regexp {^([A-Z0-9_]+)='?((.*)')?} $line \
                                _ feat __ val]} {
                            # Handle (some) shell variable substitutions
                            regsub -all {\\} $val {\\\\} val
                            regsub -all {\[} $val {\\[} val
                            regsub -all {\${(T(CL|K)_[A-Z_]+)}} $val \
                                    {[$this GetValue "\1"]} val
                            set val [subst -novariables $val]
                            $this SetValue $feat $val
                        }
                        gets $f line
                    }
                    close $f
                    break
                }
            }
        }
    } 

    method LoadCache {} {
        set fn [file join $cacheDir [string tolower \
                [$this GetValue platform_name]].tcl]
        if {[file readable $fn]} {
            if {[catch {uplevel #0 source [list $fn]} msg]} {
                Oc_Log Log "Error sourcing $fn:
	$msg" warning Oc_Config
            }
        }
    }

    method SetValue {feature value} {
        set values($feature) $value
    }

}

